@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -107,6 +107,7 @@ void AudioEngineTests::testTransportProcessing() {
 	pCoreActionController->activateLoopMode( true );
 
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 
     std::random_device randomSeed;
     std::default_random_engine randomEngine( randomSeed() );
@@ -117,7 +118,6 @@ void AudioEngineTests::testTransportProcessing() {
 	// Playing or Ready.
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
-	pAE->setState( AudioEngine::State::Testing );
 
 	// Check consistency of updated frames, ticks, and queuing
 	// position while using a random buffer size (e.g. like PulseAudio
@@ -166,6 +166,56 @@ void AudioEngineTests::testTransportProcessing() {
 
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
+
+	pAE->setState( AudioEngine::State::Ready );
+	pAE->unlock();
+
+	// Check whether all frames are covered when running playback in song mode
+	// without looping.
+	pCoreActionController->activateLoopMode( false );
+
+	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
+	resetVariables();
+	while ( nn <= nMaxCycles ) {
+		nFrames = frameDist( randomEngine );
+		pAE->incrementTransportPosition( nFrames );
+
+		if ( pAE->isEndOfSongReached( pAE->m_pTransportPosition ) ) {
+			// End of song reached
+			if ( pTransportPos->getTick() < pAE->getSongSizeInTicks() ) {
+				AudioEngineTests::throwException(
+					QString( "[testTransportProcessing] [song mode : no looping] final tick was not reached at song end. pTransportPos->getTick: [%1], pAE->getSongSizeInTicks: %2" )
+					.arg( pTransportPos->getTick() ).arg( pAE->getSongSizeInTicks() ) );
+			}
+			break;
+		}
+
+		nn++;
+		if ( nn > nMaxCycles ) {
+			AudioEngineTests::throwException(
+				QString( "[testTransportProcessing] [song mode : no looping] end of the song wasn't reached in time. pTransportPos->getFrame(): %1, pTransportPos->getDoubleTick(): %2, pTransportPos->getTickSize(): %3, pAE->getSongSizeInTicks(): %4, nMaxCycles: %5" )
+				.arg( pTransportPos->getFrame() )
+				.arg( pTransportPos->getDoubleTick(), 0, 'f' )
+				.arg( pTransportPos->getTickSize(), 0, 'f' )
+				.arg( pAE->getSongSizeInTicks(), 0, 'f' )
+				.arg( nMaxCycles ) );
+		}
+	}
+
+	pAE->reset( false );
+	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
+
+	pAE->setState( AudioEngine::State::Ready );
+	pAE->unlock();
+
+	// Check whether all frames are covered when running playback in song mode
+	// without looping.
+	pCoreActionController->activateLoopMode( true );
+
+	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
+
 	resetVariables();
 
 	float fBpm;
@@ -254,6 +304,7 @@ void AudioEngineTests::testTransportProcessingTimeline() {
 	pCoreActionController->activateLoopMode( true );
 
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 
 	// Activating the Timeline without requiring the AudioEngine to be locked.
 	auto activateTimeline = [&]( bool bEnabled ) {
@@ -279,7 +330,6 @@ void AudioEngineTests::testTransportProcessingTimeline() {
 	// Playing or Ready.
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
-	pAE->setState( AudioEngine::State::Testing );
 
 	// Check consistency of updated frames, ticks, and queuing
 	// position while using a random buffer size (e.g. like PulseAudio
@@ -391,12 +441,12 @@ void AudioEngineTests::testLoopMode() {
 	pCoreActionController->activateSongMode( true );
 
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 
 	// For this call the AudioEngine still needs to be in state
 	// Playing or Ready.
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
-	pAE->setState( AudioEngine::State::Testing );
 
 	// Check consistency of updated frames, ticks, and queuing
 	// position while using a random buffer size (e.g. like PulseAudio
@@ -449,9 +499,11 @@ void AudioEngineTests::testLoopMode() {
 		// loop mode.
 		if ( bLoopEnabled && pTransportPos->getDoubleTick() >
 			 fSongSizeInTicks * ( nLoops - 1 ) ) {
+			pAE->setState( AudioEngine::State::Ready );
 			pAE->unlock();
 			pCoreActionController->activateLoopMode( false );
 			pAE->lock( RIGHT_HERE );
+			pAE->setState( AudioEngine::State::Testing );
 		}
 		
 		nn++;
@@ -487,6 +539,7 @@ int AudioEngineTests::processTransport( const QString& sContext,
 										 long* nLastQueuingTick,
 										 double* fLastTickIntervalEnd,
 										 bool bCheckLookahead ) {
+	auto pSong = Hydrogen::get_instance()->getSong();
 	auto pAE = Hydrogen::get_instance()->getAudioEngine();
 	auto pTransportPos = pAE->getTransportPosition();
 	auto pQueuingPos = pAE->m_pQueuingPosition;
@@ -510,13 +563,13 @@ int AudioEngineTests::processTransport( const QString& sContext,
 		*nLastLookahead = nLeadLag + AudioEngine::nMaxTimeHumanize + 1;
 	}
 
-	const int nRet = pAE->updateNoteQueue( nFrames );
+	pAE->updateNoteQueue( nFrames );
 	pAE->incrementTransportPosition( nFrames );
 
-	if ( nRet != 0 ) {
+	if ( pAE->isEndOfSongReached( pAE->m_pTransportPosition ) ) {
 		// Don't check consistency at the end of the song as just the
 		// remaining frames are covered.
-		return nRet;
+		return -1;
 	}
 
 	AudioEngineTests::checkTransportPosition(
@@ -528,9 +581,10 @@ int AudioEngineTests::processTransport( const QString& sContext,
 	if ( pTransportPos->getFrame() - nFrames -
 		 pTransportPos->getFrameOffsetTempo() != *nLastTransportFrame ) {
 		AudioEngineTests::throwException(
-			QString( "[processTransport : transport] [%1] inconsistent frame update. pTransportPos->getFrame(): %2, nFrames: %3, nLastTransportFrame: %4, pTransportPos->getFrameOffsetTempo(): %5" )
+			QString( "[processTransport : transport] [%1] inconsistent frame update. pTransportPos->getFrame(): %2, nFrames: %3, nLastTransportFrame: %4, pTransportPos->getFrameOffsetTempo(): %5, pAE->m_fSongSizeInTicks: %6, pAE->m_nLoopsDone: %7" )
 			.arg( sContext ).arg( pTransportPos->getFrame() ).arg( nFrames )
-			.arg( *nLastTransportFrame ).arg( pTransportPos->getFrameOffsetTempo() ) );
+			.arg( *nLastTransportFrame ).arg( pTransportPos->getFrameOffsetTempo() )
+			.arg( pAE->m_fSongSizeInTicks ).arg( pAE->m_nLoopsDone ) );
 	}
 	*nLastTransportFrame = pTransportPos->getFrame() -
 			pTransportPos->getFrameOffsetTempo();
@@ -542,14 +596,16 @@ int AudioEngineTests::processTransport( const QString& sContext,
 	// an update has actually taken place.
 	if ( *nLastQueuingTick > 0 && nNoteQueueUpdate > 0 ) {
 		if ( pQueuingPos->getTick() - nNoteQueueUpdate !=
-			 *nLastQueuingTick ) {
+			 *nLastQueuingTick &&
+			 ! pAE->isEndOfSongReached( pQueuingPos ) ) {
 			AudioEngineTests::throwException(
-				QString( "[processTransport : queuing pos] [%1] inconsistent tick update. pQueuingPos->getTick(): %2, nNoteQueueUpdate: %3, nLastQueuingTick: %4, fTickStart: %5, fTickEnd: %6, nFrames = %7, pTransportPos: %8, pQueuingPos: %9" )
+				QString( "[processTransport : queuing pos] [%1] inconsistent tick update. pQueuingPos->getTick(): %2, nNoteQueueUpdate: %3, nLastQueuingTick: %4, fTickStart: %5, fTickEnd: %6, nFrames = %7, pTransportPos: %8, pQueuingPos: %9, pAE->m_fSongSizeInTicks: %10, pAE->m_nLoopsDone: %11" )
 				.arg( sContext ).arg( pQueuingPos->getTick() )
 				.arg( nNoteQueueUpdate ).arg( *nLastQueuingTick )
 				.arg( fTickStart, 0, 'f' ).arg( fTickEnd, 0, 'f' )
 				.arg( nFrames ).arg( pTransportPos->toQString() )
-				.arg( pQueuingPos->toQString() ) );
+				.arg( pQueuingPos->toQString() )
+				.arg( pAE->m_fSongSizeInTicks ).arg( pAE->m_nLoopsDone ));
 		}
 	}
 	*nLastQueuingTick = pQueuingPos->getTick();
@@ -592,6 +648,7 @@ void AudioEngineTests::testTransportRelocation() {
 	auto pTransportPos = pAE->getTransportPosition();
 
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 
     std::random_device randomSeed;
     std::default_random_engine randomEngine( randomSeed() );
@@ -602,7 +659,6 @@ void AudioEngineTests::testTransportRelocation() {
 	// Playing or Ready.
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
-	pAE->setState( AudioEngine::State::Testing );
 
 	// Check consistency of updated frames and ticks while relocating
 	// transport.
@@ -630,16 +686,6 @@ void AudioEngineTests::testTransportRelocation() {
 		AudioEngineTests::checkTransportPosition(
 			pTransportPos, "[testTransportRelocation] mismatch tick-based" );
 
-		if ( pAE->updateNoteQueue( pPref->m_nBufferSize ) == -1 &&
-			pAE->m_fLastTickEnd < pAE->m_fSongSizeInTicks ) {
-			AudioEngineTests::throwException(
-				QString( "[testTransportRelocation] [tick] invalid end of song: fNewTick: %1, pAE->m_fSongSizeInTicks: %2, pAE->m_fLastTickEnd: %3, transport: %4;, queuing: %5" )
-				.arg( fNewTick, 0, 'f' )
-				.arg( pAE->m_pTransportPosition->toQString() )
-				.arg( pAE->m_fSongSizeInTicks ).arg( pAE->m_fLastTickEnd )
-				.arg( pAE->m_pTransportPosition->toQString() ) );
-		}
-
 		// Frame-based relocation
 		nNewFrame = frameDist( randomEngine );
 		pAE->locateToFrame( nNewFrame );
@@ -647,14 +693,6 @@ void AudioEngineTests::testTransportRelocation() {
 		AudioEngineTests::checkTransportPosition(
 			pTransportPos, "[testTransportRelocation] mismatch frame-based" );
 
-		if ( pAE->updateNoteQueue( pPref->m_nBufferSize ) == -1 &&
-			pAE->m_fLastTickEnd < pAE->m_fSongSizeInTicks ) {
-			AudioEngineTests::throwException(
-				QString( "[testTransportRelocation] [frame] invalid end of song: nNewFrame: %1, pAE->m_fSongSizeInTicks: %2, pAE->m_fLastTickEnd: %3, transport: %4;, queuing: %5" )
-				.arg( nNewFrame ).arg( pAE->m_pTransportPosition->toQString() )
-				.arg( pAE->m_fSongSizeInTicks ).arg( pAE->m_fLastTickEnd )
-				.arg( pAE->m_pTransportPosition->toQString() ) );
-		}
 	}
 
 	pAE->reset( false );
@@ -672,6 +710,7 @@ void AudioEngineTests::testSongSizeChange() {
 	const int nTestColumn = 4;
 	
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
 	pAE->setState( AudioEngine::State::Ready );
@@ -723,6 +762,7 @@ void AudioEngineTests::testSongSizeChangeInLoopMode() {
 	pCoreActionController->activateLoopMode( true );
 
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 
 	const int nColumns = pSong->getPatternGroupVector()->size();
 
@@ -735,7 +775,6 @@ void AudioEngineTests::testSongSizeChangeInLoopMode() {
 	// Playing or Ready.
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
-	pAE->setState( AudioEngine::State::Testing );
 
 	const uint32_t nFrames = 500;
 	const double fInitialSongSize = pAE->m_fSongSizeInTicks;
@@ -779,15 +818,19 @@ void AudioEngineTests::testSongSizeChangeInLoopMode() {
 
 		nNewColumn = columnDist( randomEngine );
 
+		pAE->setState( AudioEngine::State::Ready );
 		pAE->unlock();
 		pCoreActionController->toggleGridCell( nNewColumn, 0 );
 		pAE->lock( RIGHT_HERE );
+		pAE->setState( AudioEngine::State::Testing );
 
 		checkState( QString( "toggling column [%1]" ).arg( nNewColumn ), true );
-														  
+
+		pAE->setState( AudioEngine::State::Ready );
 		pAE->unlock();
 		pCoreActionController->toggleGridCell( nNewColumn, 0 );
 		pAE->lock( RIGHT_HERE );
+		pAE->setState( AudioEngine::State::Testing );
 
 		checkState( QString( "again toggling column [%1]" ).arg( nNewColumn ), false );
 	}
@@ -810,6 +853,7 @@ void AudioEngineTests::testNoteEnqueuing() {
 	pCoreActionController->activateLoopMode( false );
 	pCoreActionController->activateSongMode( true );
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 
     std::random_device randomSeed;
     std::default_random_engine randomEngine( randomSeed() );
@@ -820,7 +864,6 @@ void AudioEngineTests::testNoteEnqueuing() {
 	// Playing or Ready.
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
-	pAE->setState( AudioEngine::State::Testing );
 
 	// Check consistency of updated frames and ticks while using a
 	// random buffer size (e.g. like PulseAudio does).
@@ -869,12 +912,8 @@ void AudioEngineTests::testNoteEnqueuing() {
 	while ( pQueuingPos->getDoubleTick() < pAE->m_fSongSizeInTicks ) {
 
 		nFrames = frameDist( randomEngine );
-		nRes = pAE->updateNoteQueue( nFrames );
+		pAE->updateNoteQueue( nFrames );
 		retrieveNotes( "song mode" );
-
-		if ( nRes == -1 ) {
-			break;
-		}
 	}
 
 	auto checkQueueConsistency = [&]( const QString& sContext ) {
@@ -1028,9 +1067,9 @@ void AudioEngineTests::testNoteEnqueuing() {
 	pCoreActionController->activateSongMode( true );
 
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
-	pAE->setState( AudioEngine::State::Testing );
 
 	nLoops = 1;
 
@@ -1066,15 +1105,15 @@ void AudioEngineTests::testNoteEnqueuing() {
 		if ( ( pTransportPos->getDoubleTick() >
 			   pAE->m_fSongSizeInTicks * nLoops + 100 ) &&
 			 pSong->getLoopMode() == Song::LoopMode::Enabled ) {
+			pAE->setState( AudioEngine::State::Ready );
+			pAE->unlock();
 			pCoreActionController->activateLoopMode( false );
+			pAE->lock( RIGHT_HERE );
+			pAE->setState( AudioEngine::State::Testing );
 		}
 
-		nRes = pAE->updateNoteQueue( nFrames );
+		pAE->updateNoteQueue( nFrames );
 		retrieveNotes( "looped song mode" );
-
-		if ( nRes == -1 ) {
-			break;
-		}
 	}
 
 	checkQueueConsistency( "looped song mode" );
@@ -1092,6 +1131,7 @@ void AudioEngineTests::testNoteEnqueuingTimeline() {
 	auto pPref = Preferences::get_instance();
 
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 
     std::random_device randomSeed;
     std::default_random_engine randomEngine( randomSeed() );
@@ -1102,7 +1142,6 @@ void AudioEngineTests::testNoteEnqueuingTimeline() {
 	// Playing or Ready.
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
-	pAE->setState( AudioEngine::State::Testing );
 	AudioEngineTests::resetSampler( __PRETTY_FUNCTION__ );
 
 	uint32_t nFrames;
@@ -1118,17 +1157,12 @@ void AudioEngineTests::testNoteEnqueuingTimeline() {
 	std::vector<std::shared_ptr<Note>> notesInSamplerQueue;
 
 	int nn = 0;
-	bool bEndOfSongReached = false;
 	while ( pTransportPos->getDoubleTick() <
 			pAE->m_fSongSizeInTicks ) {
 
 		nFrames = frameDist( randomEngine );
 
-		if ( ! bEndOfSongReached ) {
-			if ( pAE->updateNoteQueue( nFrames ) == -1 ) {
-				bEndOfSongReached = true;
-			}
-		}
+		pAE->updateNoteQueue( nFrames );
 
 		// Add freshly enqueued notes.
 		AudioEngineTests::mergeQueues( &notesInSongQueue,
@@ -1202,12 +1236,12 @@ void AudioEngineTests::testHumanization() {
 	pCoreActionController->activateSongMode( true );
 
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 
 	// For reset() the AudioEngine still needs to be in state
 	// Playing or Ready.
 	pAE->reset( false );
 	pAE->m_fSongSizeInTicks = pSong->lengthInTicks();
-	pAE->setState( AudioEngine::State::Testing );
 
 	// Rolls playback from beginning to the end of the song and
 	// captures all notes added to the Sampler.
@@ -1237,15 +1271,10 @@ void AudioEngineTests::testHumanization() {
 		}
 		
 		int nn = 0;
-		bool bEndOfSongReached = false;
 		while ( pTransportPos->getDoubleTick() < pAE->m_fSongSizeInTicks ||
 				pAE->getEnqueuedNotesNumber() > 0 ) {
 
-			if ( ! bEndOfSongReached ) {
-				if ( pAE->updateNoteQueue( nFrames ) == -1 ) {
-					bEndOfSongReached = true;
-				}
-			}
+			pAE->updateNoteQueue( nFrames );
 
 			pAE->processAudio( nFrames );
 			
@@ -1309,10 +1338,12 @@ void AudioEngineTests::testHumanization() {
 	// positions but velocity, pan, leag&lag, and note key as well as
 	// note octave are all customized. Check whether these
 	// customizations reach the Sampler.
+	pAE->setState( AudioEngine::State::Ready );
 	pAE->unlock();
 	pCoreActionController->toggleGridCell( 0, 0 );
 	pCoreActionController->toggleGridCell( 0, 1 );
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 
 	std::vector<std::shared_ptr<Note>> notesCustomized;
 	getNotes( &notesCustomized );
@@ -1372,10 +1403,12 @@ void AudioEngineTests::testHumanization() {
 	// expected standard deviation.
 	//
 	// Switch back to pattern 1
+	pAE->setState( AudioEngine::State::Ready );
 	pAE->unlock();
 	pCoreActionController->toggleGridCell( 0, 1 );
 	pCoreActionController->toggleGridCell( 0, 0 );
 	pAE->lock( RIGHT_HERE );
+	pAE->setState( AudioEngine::State::Testing );
 
 	auto checkHumanization = [&]( double fValue, std::vector<std::shared_ptr<Note>>* pNotes ) {
 
@@ -1646,13 +1679,13 @@ void AudioEngineTests::checkAudioConsistency( const std::vector<std::shared_ptr<
 						
 						const int nSampleFrames =
 							ppNewNote->get_instrument()->get_component( nn )
-							->get_layer( pSelectedLayer->SelectedLayer )
+							->get_layer( pSelectedLayer->nSelectedLayer )
 							->get_sample()->get_frames();
 						const double fExpectedFrames =
-							std::min( static_cast<double>(pSelectedLayer->SamplePosition) +
+							std::min( static_cast<double>(pSelectedLayer->fSamplePosition) +
 									  fPassedFrames,
 									  static_cast<double>(nSampleFrames) );
-						if ( std::abs( ppNewNote->get_layer_selected( nn )->SamplePosition -
+						if ( std::abs( ppNewNote->get_layer_selected( nn )->fSamplePosition -
 									   fExpectedFrames ) > 1 ) {
 							AudioEngineTests::throwException(
 								QString( "[checkAudioConsistency] [%4] glitch in audio render. Diff: %9\nPre: %1\nPost: %2\nwith passed frames: %3, nSampleFrames: %5, fExpectedFrames: %6, sample sampleRate: %7, driver sampleRate: %8\n" )
@@ -1662,7 +1695,7 @@ void AudioEngineTests::checkAudioConsistency( const std::vector<std::shared_ptr<
 								.arg( nSampleFrames ).arg( fExpectedFrames, 0, 'f' )
 								.arg( ppOldNote->getSample( nn )->get_sample_rate() )
 								.arg( nSampleRate )
-								.arg( ppNewNote->get_layer_selected( nn )->SamplePosition -
+								.arg( ppNewNote->get_layer_selected( nn )->fSamplePosition -
 									  fExpectedFrames, 0, 'g', 30 ) );
 						}
 					}
@@ -1771,10 +1804,12 @@ void AudioEngineTests::toggleAndCheckConsistency( int nToggleColumn, int nToggle
 		nOldColumn = pTransportPos->getColumn();
 		fPrevTempo = pTransportPos->getBpm();
 		fPrevTickSize = pTransportPos->getTickSize();
-	
+
+		pAE->setState( AudioEngine::State::Ready );
 		pAE->unlock();
 		pCoreActionController->toggleGridCell( nToggleColumn, nToggleRow );
 		pAE->lock( RIGHT_HERE );
+		pAE->setState( AudioEngine::State::Testing );
 
 		const QString sNewContext =
 			QString( "toggleAndCheckConsistency::toggleAndCheck : %1 : toggling (%2,%3)" )
